/////////////////////////////////////////////////////////////////////////////
//
// PocketFrog - The Game Library for Pocket PC Devices
// Copyright 2002  Thierry Tremblay
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Thierry Tremblay makes no representations
// about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
//////////////////////////////////////////////////////////////////////////////

#include "framework.h"

//////////////////////////////////////////////////////////////////////////////
//
// Game
//
//////////////////////////////////////////////////////////////////////////////

Game::Game()
{
    m_config.szWindowName = _T( "PocketFrog" );
 	m_config.desktopZoom  = 0;
	m_config.frameRate    = 30;

	m_display=NULL;
}



Game::~Game()
{
}

void Game::FatalError( LPCTSTR msg )
{
    if (m_hWnd) DestroyWindow();

    ::MessageBox( 0, msg, _T("Fatal Error"), MB_OK);
    
	exit(-1);
}



bool Game::Init()
{
	m_bShutdown = false;

	GetWndClassInfo().m_wc.style = CS_HREDRAW | CS_VREDRAW;
	GetWndClassInfo().m_wc.hbrBackground = 0;

    // Create the window
	// PocketHal resizes the window after initialize
    rect_t r;
#if defined(WIN32_PLATFORM_PSPC)
	SetRect(&r, 0, 0, ::GetSystemMetrics( SM_CXSCREEN ), ::GetSystemMetrics( SM_CYSCREEN ) );
#else 
	switch (m_config.parameters.m_orientation) {
	case ORIENTATION_PORTRAIT:
		SetRect(&r, 0, 0, 239, 319);
		break;
	case ORIENTATION_LANDSCAPE:
		SetRect(&r, 0, 0, 319, 239);
		break;
	default:
		SetRect(&r, 0, 0, ::GetSystemMetrics( SM_CXSCREEN ), ::GetSystemMetrics( SM_CYSCREEN ) );
		break;
	}
#endif	

    DWORD style = WS_VISIBLE;
    DWORD exStyle = WS_EX_TOPMOST;

    if (!Create( 0, r, m_config.szWindowName, style, exStyle ))	{
		FatalError( _T("Window creation failed") );
        return false;
	}

    // Init display
	m_display=Display::Create();
	if (m_display == 0) {
		FatalError( _T("Could not create display device") );
        return false;
	}
	if (m_display->Initialize(m_hWnd, m_config.parameters) == false) {
		FatalError( _T("Could not initialize display device") );
        return false;
	}

    // Init input
	GetKeyList(&m_keylist, m_config.parameters.m_orientation);

	ShowWindow(SW_SHOW);
	UpdateWindow();
    
    return true;
}



bool Game::Run()
{
	ShowTaskbar(false);

    // Initialization
    if (!Init())
	{
		FatalError( _T("Game::Init() failed") );
        return false;
	}

    // Game will start...
    if (!GameInit())
    {
        Shutdown();
        return false;
    }
    
    // Main loop
    for(;;)
    {
        MSG msg;

        if (::PeekMessage( &msg, 0, 0, 0, PM_REMOVE ))
        {
            if (msg.message == WM_QUIT || m_bShutdown) {
                break;
			}
            
            ::TranslateMessage( &msg );
            ::DispatchMessage( &msg );
        }
        else
        {
			GameLoop();
        }
    }

	// Game has ended
	GameEnd();

	// Clean up
	m_display->Shutdown();
	delete m_display;

	ShowTaskbar(true);
	::DestroyWindow(m_hWnd);

    return true;
}


void Game::ShowTaskbar( bool bShow )
{
	HWND hWndTaskbar = ::FindWindow(TEXT("HHTaskbar"), NULL);
	if (bShow) {
		::ShowWindow(hWndTaskbar, SW_SHOW);
#if defined (WIN32_PLATFORM_PSPC)
		SHFullScreen(m_hWnd, SHFS_SHOWSTARTICON | SHFS_SHOWTASKBAR | SHFS_SHOWSIPBUTTON);		
#endif
	} else {
		::ShowWindow(hWndTaskbar, SW_HIDE);
#if defined (WIN32_PLATFORM_PSPC)
		SHFullScreen(m_hWnd, SHFS_HIDESTARTICON | SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON);
#endif
	}
}

void Game::Shutdown()
{
	m_bShutdown=true;
}



//////////////////////////////////////////////////////////////////////////////
//
// Default implementation of callbacks
//
//////////////////////////////////////////////////////////////////////////////

bool Game::GameInit()
{
    return true;
}

void Game::GameEnd()
{
}

void Game::GameLoop()
{
}

void Game::ButtonDown( int button )
{
    Shutdown();
}

void Game::ButtonUp( int button )
{
}

void Game::StylusDown( point_t p )
{
	Shutdown();
}

void Game::StylusMove( point_t p )
{
}

void Game::StylusUp( point_t p )
{
}



//////////////////////////////////////////////////////////////////////////////
//
// Message Handlers
//
//////////////////////////////////////////////////////////////////////////////

LRESULT Game::OnClose( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
    Shutdown();
    return 0;
}



LRESULT Game::OnCreate( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
    return 0;
}



LRESULT Game::OnDestroy( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
	Shutdown();
    return 0;
}



LRESULT Game::OnKeyDown( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
    if (!(lparam & 0x40000000)) // Prevent auto-repeat
    {
        s32 button = wparam;
		ButtonDown( button );
	}
    
    if (wparam == VK_ESCAPE)
        Shutdown();

    return 0;
}



LRESULT Game::OnKeyUp( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
    s32 button = wparam;
    ButtonUp( button );

    return 0;
}



LRESULT Game::OnMouseEvent( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
	point_t P;
    P.x=LOWORD(lparam);
	P.y=HIWORD(lparam);

    //m_input->DeviceToLogical( P );

    switch(msg)
    {
    case WM_LBUTTONDOWN: StylusDown( P ); break;
    case WM_MOUSEMOVE:   StylusMove( P ); break;
    case WM_LBUTTONUP:   StylusUp( P );   break;
    }
    
    return 0;
}

LRESULT Game::OnPaint( UINT msg, WPARAM wparam, LPARAM lparam, BOOL& bHandled )
{
    PAINTSTRUCT ps;
    
    BeginPaint( &ps );
    EndPaint( &ps );

	return 0;
}


